package amoba

import scala.util.{Random}
import swing.{SimpleSwingApplication, MainFrame, Panel, event}
import event.{KeyPressed}
import scala.swing.event.MousePressed
import scala.swing.event.MousePressed
import scala.swing.event.MouseClicked
import scala.collection.mutable.ListBuffer
import sun.java2d.SunGraphics2D
import java.awt.image.BufferedImage
import javax.imageio.ImageIO
import java.io.File
import java.awt.Image
import scala.swing.Label
import scala.Application

object Application extends SimpleSwingApplication {
	import event.Key._
	import java.awt.{Dimension, Graphics2D, Rectangle}
	import java.awt.{Color}
	import java.awt.event.{ActionEvent}
	import javax.swing.{Timer, AbstractAction}
	import CellType._

	override def top = frame
	
	// kepek a kor es x rajzolasahoz
	val firstImage: Image = ImageIO.read(new File("bin\\img\\x.gif"))
	val secondImage: Image = ImageIO.read(new File("bin\\img\\o.gif"))

	// fo frame, ez az alkalmazas foablaka
	val frame = new MainFrame {
		title = "Scala Amoba"
		  
		// a foablak egyetlen eleme a panel, amin a jatekmezo lesz kirajzolva
		contents = mainPanel

		lazy val mainPanel = new Panel {
			focusable = true
			background = Color.white
			preferredSize = new Dimension(400, 400)
			
			// Panel metodusa, ami minden grafikus valtozasnal lefut
			override def paint(g: Graphics2D) {
				g.setColor(Color.white)
				g.fillRect(0, 0, size.width, size.height)
				drawField(g)
			}

			// billentyuzet es eger akciokra reagal a program
			listenTo(keys,mouse.clicks)

			// esemenykezelok hozzaadasa
			reactions += {
				case KeyPressed(_, key, _, _) =>
				onKeyPress(key)
				case e: MouseClicked => onMouseClick(e)
			}
		}

		// escape-re kilep a programbol, mas gombra nem csinal semmit
		def onKeyPress(keyCode: event.Key.Value) = keyCode match {
			case Escape => System.exit(1)
			case _ =>
		}
	}
	
	// egerkattintas esemenykezeloje
	def onMouseClick(e: MouseClicked) {
	  if (!game.isOver) { // csak akkor valtozik valami, ha nincs vege a jateknak
		  // az egerkattintas helyebol kiszamoljuk, hogy melyik mezore jott a kattintas	
	      val cell_x = (e.point.x - 1) / 20
	      val cell_y = (e.point.y - 1) / 20
	      
	      // a jatekmenet donti el, hogy mi valtozik a feluleten
	      val cellType: CellType = game.step(cell_x, cell_y)
	      cellType match {
		    case CellType.FirstPlayer => updateField() // elso es masodik jatekos eseten frissitjuk a tablat
		    case CellType.SecondPlayer => updateField()
		    case CellType.GameOver => gameOver(cell_x, cell_y)
		    case _ => /*System.out.println(cell_x + " " + cell_y)*/
			}
		}
	}

	def updateField() = {
	  frame.repaint
	}
	
	// ha a jatek veget ert, konzolba kiirjuk a gyoztest es frissitjuk a tablat az utolso lepes alapjan
	def gameOver(x: Int, y: Int) = {
	  val player = game.playerOn
	  var text = "Winner: "
	  if (player == 1)
	    text += "First player"
	  else
	    text += "Second player"
	  System.out.println("GAME OVER, " + text)
	  updateField()
	}
	
	var cellArray = Array.ofDim[Rectangle](20,20)
	
	def drawField(g: Graphics2D) {
		for (i <- 0 to 19) {
			for (j <- 0 to 19) {
				g.setColor(Color.black)
				cellArray(i)(j) = drawBlock(i, j)
			}
		}
		
		game.grid.usedCells.foreach{cell: Cell =>
		  drawCell(cell)} 
	  
		// beagyazott fuggveny, kezdeti kirajzolas
		def drawBlock(x: Int, y: Int): Rectangle = {
			var rec: Rectangle = new Rectangle(x * 20 + 1, y * 20 + 1, 18, 18)
			g.draw(rec)
			g.setColor(Color.white);
			g.fillRect(x * 20 + 1, y * 20 + 1, 18, 18)
			return rec
		}
		
		// beagyazott fuggveny, egy jatekos lepese utani frissites
		def drawCell(cell: Cell): Rectangle = {
		  var rec: Rectangle = new Rectangle(cell.x * 20 + 1, cell.y * 20 + 1, 18, 18)
		  g.setColor(Color.black)
		  g.draw(rec)
		  if (cell.player == 1) {
			g.drawImage(firstImage, cell.x * 20 + 1, cell.y * 20 + 1, null)
		  }
			else if (cell.player == 2) {
			  g.drawImage(secondImage, cell.x * 20 + 1, cell.y * 20 + 1, null)
			}
			return rec
		}
		
	}
	
	// jatek objektuma, tarsobjektumon keresztul hozzuk letre
	var game = Game.newGame

}

// a jatek singleton objektuma (habar nincs letiltva a Game osztaly peldanyositasa)
object Game {
	def newGame: Game = {
		new Game(Grid.newGrid)
	}
}

// jatekmenetet reprezentalo osztaly
class Game(val grid: Grid) {
  import CellType._
  
  // aktualis jatekos
  var playerOn: Int = 1
  var isOver = false
  
  // megnezi, hogy foglalt-e a mezo
  def isCellUsed(cell: Cell): Boolean = {
    grid.usedCells.foreach { usedCell: Cell =>
    	if (cell == usedCell) return true
    }
    return false
  }
  
  // egy lepes a jatekban
  def step(x: Int, y: Int): CellType = {
    val currentPlayer = playerOn
    val cell = new Cell(x, y, currentPlayer)
    if (isCellUsed(cell))
    	return CellType.UsedCell
    	
    grid.usedCells += cell
    (grid.cellArray(x)(y)).player = currentPlayer
    if (isGameOver(cell)) {
      isOver = true
      return CellType.GameOver
    }
    changePlayer
    return playerToCellType(currentPlayer)
  }
  
  // elvegzi a megfelelo mezoellenorzeseket
  def isGameOver(cell: Cell): Boolean = {
    val player = cell.player
    val x = cell.x
    val y = cell.y
    return isHorizontalOver(x, y, player) || isVerticalOver(x, y, player) || isTopDownOver(x, y, player) || isBottomUpOver(x, y, player)
  }
  
  // vizszintesen van-e 5 osszefuggo
  def isHorizontalOver(x: Int, y: Int, player: Int): Boolean = {
    var samePlayerCells = 0
    var i = x
    var j = y
    var isCellSamePlayer = true
    while (i >= 0 && isCellSamePlayer) {
      var cell = grid.cellArray(i)(j)
      if (cell.player == player) samePlayerCells += 1
      else isCellSamePlayer = false
      i -= 1
    }
    i = x
    if (i < 19) {
    	i = x + 1
    	isCellSamePlayer = true
    	while (i <= 19 && isCellSamePlayer) {
    	  var cell = grid.cellArray(i)(j)
    		if (cell.player == player) samePlayerCells += 1
    		else isCellSamePlayer = false
    		i += 1
    	}
    }
    return samePlayerCells == 5
  }
  
  // fuggolegesen van-e 5 osszefuggo
  def isVerticalOver(x: Int, y: Int, player: Int): Boolean = {
    var samePlayerCells = 0
    var i = x
    var j = y
    var isCellSamePlayer = true
    while (j >= 0 && isCellSamePlayer) {
      var cell = grid.cellArray(i)(j)
      if (cell.player == player) samePlayerCells += 1
    		else isCellSamePlayer = false
      j -= 1
    }
    j = y
    if (j < 19) {
    	j = y + 1
    	isCellSamePlayer = true
    	while (j <= 19 && isCellSamePlayer) {
    	  var cell = grid.cellArray(i)(j)
    		if (cell.player == player) samePlayerCells += 1
    		else isCellSamePlayer = false
    		j += 1
    	}
    }
    return samePlayerCells == 5
  }
  
  // bal-fent -> jobb-lent atloban van-e 5 osszefuggo
  def isTopDownOver(x: Int, y: Int, player: Int): Boolean = {
    var samePlayerCells = 0
    var i = x
    var j = y
    var isCellSamePlayer = true
    while (i >= 0 && j >= 0 && isCellSamePlayer) {
      var cell = grid.cellArray(i)(j)
      if (cell.player == player) samePlayerCells += 1
    		else isCellSamePlayer = false
      i -= 1
      j -= 1
    }
    i = x
    j = y
    if (j < 19 && x < 19) {
    	i = x + 1
    	j = y + 1
    	isCellSamePlayer = true
    	while (i <= 19 && j <= 19 && isCellSamePlayer) {
    	  var cell = grid.cellArray(i)(j)
    		if (cell.player == player) samePlayerCells += 1
    		else isCellSamePlayer = false
    		i += 1
    		j += 1
    	}
    }
    return samePlayerCells == 5
  }
  
  // bal-lent -> jobb-fent atloban van-e 5 osszefuggo
  def isBottomUpOver(x: Int, y: Int, player: Int): Boolean = {
    var samePlayerCells = 0
    var i = x
    var j = y
    var isCellSamePlayer = true
    while (i <= 19 && j >= 0 && isCellSamePlayer) {
      var cell = grid.cellArray(i)(j)
      if (cell.player == player) samePlayerCells += 1
    		else isCellSamePlayer = false
      i += 1
      j -= 1
    }
    i = x
    j = y
    if (j < 19 && i > 0) {
    	i = x - 1
    	j = y + 1
    	isCellSamePlayer = true
    	while (i >= 0 && j <= 19 && isCellSamePlayer) {
    	  var cell = grid.cellArray(i)(j)
    		if (cell.player == player) samePlayerCells += 1
    		else isCellSamePlayer = false
    		i -= 1
    		j += 1
    	}
    }
    return samePlayerCells == 5
  }
  
  // jatekosvaltas
  	def changePlayer = {
		  if (playerOn == 1) playerOn = 2
		  else playerOn = 1
  	}
  	
  	// jatekos (Int) -> mezotipus megfeleltetes
  	def playerToCellType(player: Int): CellType = {
  	  if (player == 1) return CellType.FirstPlayer
  	  else return CellType.SecondPlayer
  	}
}

object Grid {
	def newGrid: Grid = {
		new Grid()
	}
}

// racs, hogy konnyebben el tudjuk donteni, hogy egy mezo foglalt-e
class Grid {
	var usedCells = ListBuffer[Cell]();
	var cellArray = Array.ofDim[Cell](20,20)
	
	for (i <- 0 to 19) {
			for (j <- 0 to 19) {
				cellArray(i)(j) = new Cell(i,j,0)
			}
		}
}

// egy mezo a racson, tartalmazza, hogy melyik jatekose a mezo
class Cell(val x: Int, val y: Int, var player: Int) {
  def ==(cell: Cell) = cell.x == x && cell.y == y
}

class Position(val x: Int, val y: Int) {

def ==(pos: Position) = pos.x == x && pos.y == y

def +(move: Tuple2[Int, Int]) = new Position(x + move._1, y + move._2)

}

// enumeralt tipus a mezotipussal a biztonsagosabb ellenorzeshez
object CellType extends Enumeration {
  type CellType = Value
  val FirstPlayer, SecondPlayer, UsedCell, GameOver = Value
}
